#ifndef __mRay_H__
#define __mRay_H__

/*! 
    \file mRay.h
    \brief Ray class.
    \author Douglas H. Cox
*/

//============================================================================
//    REQUIRED HEADERS
//============================================================================
#include "mVec3.h"

//============================================================================
//    DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS / CLASS FORWARDS
//============================================================================
class mOCS;

//============================================================================
//    FUNCTION PROTOTYPES
//============================================================================

//============================================================================
//    CLASS PROTOTYPES
//============================================================================

//! Ray class.
class MMATH_DLL mRay
{
public:
  mVec3 pos; //!< position of the ray. 
  mVec3 dir; //!< normalized direction the ray points.
  float len; //!< length of the ray. if = -1, ray is infinite.

  //! Default constructor
  /*! No initialization is done here.  */
  mRay() 
  {
  }

  //! Copy constructor
  mRay(const mRay& src) : pos(src.pos),
                          dir(src.dir),
                          len(src.len)
  {
  }

  //! Constructor from position, direction, and length.
  mRay(const mVec3& position, 
       const mVec3& direction,
       const float  length) : pos(position),
                              dir(direction),
                              len(length)
  {
  }

  //! Constructor from two points.
  mRay(const mVec3& startPoint, const mVec3& endPoint)
  {
    set(startPoint, endPoint);
  }

  //! Assignment
  void set(const mRay& src)
  {
    pos.set(src.pos);
    dir.set(src.dir);
    len = src.len;
  }
  
  //! Assignment from position, direction, and length.
  void set(const mVec3& position, 
           const mVec3& direction,
           const float  length)
  {
    pos.set(position);
    dir.set(direction);
    len = length;
  }

  //! Assignment from two points.
  void set(const mVec3& startPoint, const mVec3& endPoint);

  //! Returns the starting point for this ray.
  void getStartPoint(mVec3& start) const
  {
    start.set(pos);
  }

  //! Returns the ending point for this ray.
  /*! \todo Check the case when len == -1. */
  void getEndPoint(mVec3& end) const;

  //! Returns true if the two rays are the same.
  bool operator == (const mRay& ray) const;

  //! Returns true if the two rays are not the same.
  bool operator != (const mRay& ray) const
  {
    return !(operator==(ray));
  }

  //! Transforms the ray into the OCS.
  void xformTo(const mRay& ray, const mOCS& ocs);

  //! Transforms the ray from the OCS.
  void xformFrom(const mRay& ray, const mOCS& ocs);

 
};


//****************************************************************************
//**    END HEADER mRay.h
//****************************************************************************
#endif // __mRay_H__
